package com.mortals.xhx.module.staff.web;

import com.mortals.framework.exception.AppException;
import com.mortals.framework.web.BaseCRUDJsonBodyMappingController;
import com.mortals.xhx.base.system.param.service.ParamService;
import com.mortals.xhx.common.code.CareTypeEnum;
import com.mortals.xhx.common.code.SendStatusEnum;
import com.mortals.xhx.module.dept.model.DeptEntity;
import com.mortals.xhx.module.dept.model.DeptQuery;
import com.mortals.xhx.module.dept.service.DeptService;
import com.mortals.xhx.module.job.model.JobEntity;
import com.mortals.xhx.module.job.model.JobQuery;
import com.mortals.xhx.module.job.service.JobService;
import com.mortals.xhx.module.staff.model.StaffEntity;
import com.mortals.xhx.module.staff.model.StaffQuery;
import com.mortals.xhx.module.staff.service.StaffService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.ObjectUtils;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.bind.annotation.*;
import com.mortals.framework.model.Context;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.mortals.framework.web.BaseCRUDJsonBodyMappingController;
import com.mortals.xhx.module.staff.model.StaffCareEntity;
import com.mortals.xhx.module.staff.service.StaffCareService;
import org.apache.commons.lang3.ArrayUtils;
import com.mortals.framework.util.StringUtils;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import com.alibaba.fastjson.JSONObject;

import java.util.Arrays;

import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import static com.mortals.framework.ap.SysConstains.*;

/**
 * 员工关怀信息
 *
 * @author zxfei
 * @date 2023-04-07
 */
@RestController
@RequestMapping("staff/care")
public class StaffCareController extends BaseCRUDJsonBodyMappingController<StaffCareService, StaffCareEntity, Long> {

    @Autowired
    private ParamService paramService;

    @Autowired
    private StaffService staffService;
    @Autowired
    private DeptService deptService;
    @Autowired
    private JobService jobService;

    public StaffCareController() {
        super.setModuleDesc("员工关怀信息");
    }

    @Override
    protected void init(Map<String, Object> model, Context context) {
        this.addDict(model, "careType", CareTypeEnum.getEnumMap());
        this.addDict(model, "sendStatus", SendStatusEnum.getEnumMap());
        this.addDict(model, "deptId", deptService.find(new DeptQuery()).stream().collect(Collectors.toMap(x -> x.getId().toString(), y -> y.getDeptName())));
        this.addDict(model, "jobId", jobService.find(new JobQuery()).stream().collect(Collectors.toMap(x -> x.getId().toString(), y -> y.getJobName())));
        this.addDict(model, "staffId", staffService.find(new StaffQuery()).stream().collect(Collectors.toMap(x -> x.getId().toString(), y -> y.getName())));

        super.init(model, context);
    }


    @Override
    protected void saveBefore(StaffCareEntity entity, Map<String, Object> model, Context context) throws AppException {
        if (!ObjectUtils.isEmpty(entity.getJobId())) {
            JobEntity jobEntity = jobService.get(entity.getJobId(), context);
            entity.setJobName(jobEntity == null ? "" : jobEntity.getJobName());
        }

        if (!ObjectUtils.isEmpty(entity.getStaffId())) {
            StaffEntity staffEntity = staffService.getCache(entity.getStaffId().toString());
            entity.setStaffName(staffEntity == null ? "" : staffEntity.getName());
        }
        if (!ObjectUtils.isEmpty(entity.getDeptId())) {
            DeptEntity deptEntity = deptService.get(entity.getDeptId(), context);
            entity.setDeptName(deptEntity == null ? "" : deptEntity.getDeptName());
        }

    }
}